*$ CREATE SOURCE.FOR
*COPY SOURCE
*
*=== source ===========================================================*
*
      SUBROUTINE SOURCE ( NOMORE )

      INCLUDE '(DBLPRC)'
      INCLUDE '(DIMPAR)'
      INCLUDE '(IOUNIT)'
*
*----------------------------------------------------------------------*
*                                                                      *
*     Copyright (C) 1990-2010      by    Alfredo Ferrari & Paola Sala  *
*     All Rights Reserved.                                             *
*                                                                      *
*                                                                      *
*     New source for FLUKA9x-FLUKA20xy:                                *
*                                                                      *
*     Created on 07 January 1990   by    Alfredo Ferrari & Paola Sala  *
*                                                   Infn - Milan       *
*                                                                      *
*     Last change on  17-Oct-10    by    Alfredo Ferrari               *
*                                                                      *
*  This is just an example of a possible user written source routine.  *
*  note that the beam card still has some meaning - in the scoring the *
*  maximum momentum used in deciding the binning is taken from the     *
*  beam momentum.  Other beam card parameters are obsolete.            *
*                                                                      *
*       Output variables:                                              *
*                                                                      *
*              Nomore = if > 0 the run will be terminated              *
*                                                                      *
*----------------------------------------------------------------------*
*
      INCLUDE '(BEAMCM)'
      INCLUDE '(FHEAVY)'
      INCLUDE '(FLKSTK)'
      INCLUDE '(IOIOCM)'
      INCLUDE '(LTCLCM)'
      INCLUDE '(PAPROP)'
      INCLUDE '(SOURCM)'
      INCLUDE '(SUMCOU)'
*
      LOGICAL LFIRST
*
      SAVE LFIRST
      DATA LFIRST / .TRUE. /
      PARAMETER (numb=42)
      DOUBLE PRECISION x(numb), y(numb)
*======================================================================*
*                                                                      *
*                 BASIC VERSION                                        *
*                                                                      *
*======================================================================*
      NOMORE = 0
*  +-------------------------------------------------------------------*
*  |  First call initializations:
      IF ( LFIRST ) THEN
*  |  *** The following 3 cards are mandatory ***
         TKESUM = ZERZER
         LFIRST = .FALSE.
         LUSSRC = .TRUE.
*  |  *** User initialization ***
      END IF
*  |
*  +-------------------------------------------------------------------*
*  Push one source particle to the stack. Note that you could as well
*  push many but this way we reserve a maximum amount of space in the
*  stack for the secondaries to be generated
*  Npflka is the stack counter: of course any time source is called it
*  must be =0
      NPFLKA = NPFLKA + 1
*  Wt is the weight of the particle
      WTFLK  (NPFLKA) = ONEONE
      WEIPRI = WEIPRI + WTFLK (NPFLKA)
*  Particle type (1=proton.....). Ijbeam is the type set by the BEAM
*  card
*  +-------------------------------------------------------------------*
*  |  (Radioactive) isotope:
      IF ( IJBEAM .EQ. -2 .AND. LRDBEA ) THEN
         IARES  = IPROA
         IZRES  = IPROZ
         IISRES = IPROM
         CALL STISBM ( IARES, IZRES, IISRES )
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
*  |
*  +-------------------------------------------------------------------*
*  |  Heavy ion:
      ELSE IF ( IJBEAM .EQ. -2 ) THEN
         IJHION = IPROZ  * 1000 + IPROA
         IJHION = IJHION * 100 + KXHEAV
         IONID  = IJHION
         CALL DCDION ( IONID )
         CALL SETION ( IONID )
         ILOFLK (NPFLKA) = IJHION
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |  Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
*  |
*  +-------------------------------------------------------------------*
*  |  Normal hadron:
      ELSE
         IONID = IJBEAM
         ILOFLK (NPFLKA) = IJBEAM
*  |  Flag this is prompt radiation
         LRADDC (NPFLKA) = .FALSE.
*  |  Group number for "low" energy neutrons, set to 0 anyway
         IGROUP (NPFLKA) = 0
      END IF
*  |
*  +-------------------------------------------------------------------*
*  From this point .....
*  Particle generation (1 for primaries)
      LOFLK  (NPFLKA) = 1
*  User dependent flag:
      LOUSE  (NPFLKA) = 0
*  No channeling:
      LCHFLK (NPFLKA) = .FALSE.
      DCHFLK (NPFLKA) = ZERZER
*  User dependent spare variables:
      DO 100 ISPR = 1, MKBMX1
         SPAREK (ISPR,NPFLKA) = ZERZER
 100  CONTINUE
*  User dependent spare flags:
      DO 200 ISPR = 1, MKBMX2
         ISPARK (ISPR,NPFLKA) = 0
 200  CONTINUE
*  Save the track number of the stack particle:
      ISPARK (MKBMX2,NPFLKA) = NPFLKA
      NPARMA = NPARMA + 1
      NUMPAR (NPFLKA) = NPARMA
      NEVENT (NPFLKA) = 0
      DFNEAR (NPFLKA) = +ZERZER
*  ... to this point: don't change anything
*  Particle age (s)
      AGESTK (NPFLKA) = +ZERZER
      AKNSHR (NPFLKA) = -TWOTWO
*  Kinetic energy of the particle (GeV)
***********************************************************************
*                                  Changes                            *
***********************************************************************
*      TKEFLK (NPFLKA) = SQRT ( PBEAM**2 + AM (IONID)**2 ) - AM (IONID)
c    Energy bin x -in keV
      rNorm = -1.
      x(1) = 1.
      y(1) = 3.
      x(2) = 1.3
      y(2) = 9.
      x(3) = 1.8
      y(3) = 13.
      x(4) = 1.9
      y(4) = 3.
      x(5) = 5.
      y(5) = 7.5
      x(6) = 7.38
      y(6) = 8.
      x(7) = 7.41
      y(7) = 11.
      x(8) = 7.44
      y(8) = 8.
      x(9) = 8.31
      y(9) = 7.7
      x(10) = 8.33
      y(10) = 15.
      x(11) = 8.35
      y(11) = 7.7
      x(12) = 8.37
      y(12) = 7.7
      x(13) = 8.39
      y(13) = 72.
      x(14) = 8.41
      y(14) = 7.7
      x(15) = 9.48
      y(15) = 7.3
      x(16) = 9.51
      y(16) = 13.
      x(17) = 9.54
      y(17) = 7.3
      x(18) = 9.67
      y(18) = 7.
      x(19) = 9.69
      y(19) = 50.
      x(20) = 9.71
      y(20) = 7.
      x(21) = 9.79
      y(21) = 7.
      x(22) = 9.81
      y(22) = 15.
      x(23) = 9.83
      y(23) = 7.
      x(24) = 9.93
      y(24) = 7.
      x(25) = 9.95
      y(25) = 22.
      x(26) = 9.97
      y(26) = 7.
      x(27) = 10.17
      y(27) = 7.
      x(28) = 10.23
      y(28) = 5.
      x(29) = 11.29
      y(29) = 4.8
      x(30) = 11.31
      y(30) = 11.
      x(31) = 11.33
      y(31) = 4.8
      x(32) = 13.
      y(32) = 3.8
      x(33) = 19.
      y(33) = 2.6
      x(34) = 23.
      y(34) = 1.8
      x(35) = 28.
      y(35) = 1.05
      x(36) = 32.
      y(36) = 0.48
      x(37) = 33.
      y(37) = 0.33
      x(38) = 34.
      y(38) = 0.19
      x(39) = 34.5
      y(39) = 0.1
      x(40) = 34.75
      y(40) = 0.06
      x(41) = 34.9
      y(41) = 0.03
      x(42) = 35.
      y(42) = 0.008
c
      DO 111 n = 1,numb
       IF (rNorm .LE. y(n)) THEN
        rNorm = y(n)
       END IF
 111  CONTINUE
c
  10  CONTINUE
c Energy in keV
      rE = FLRNDM() * (35.-1.) + 1.
      DO 11 n = 1,numb-1
       IF (rE.GT.(x(n)).AND.(rE.LE.x(n+1))) THEN
         rA = ( y(n+1) - y(n) ) / ( x(n+1) - x(n) )
         rB = y(n) - rA * x(n)
       END IF
      rR = rA * rE + rB
  11  CONTINUE
      IF (rNorm*FLRNDM() .GT. rR) GOTO 10
c Energies from keV to GeV
      TKEFLK (NPFLKA) = rE * 1.0E-6
***********************************************************************
*        The end of the particle spectrum generator procedure         *
***********************************************************************
*  Particle momentum
*      PMOFLK (NPFLKA) = PBEAM
      PMOFLK (NPFLKA) = SQRT ( TKEFLK (NPFLKA) * ( TKEFLK (NPFLKA)
     &                       + TWOTWO * AM (IONID) ) )
*  Cosines (tx,ty,tz)
      TXFLK  (NPFLKA) = UBEAM
      TYFLK  (NPFLKA) = VBEAM
      TZFLK  (NPFLKA) = WBEAM
*     TZFLK  (NPFLKA) = SQRT ( ONEONE - TXFLK (NPFLKA)**2
*    &                       - TYFLK (NPFLKA)**2 )
*  Polarization cosines:
      TXPOL  (NPFLKA) = -TWOTWO
      TYPOL  (NPFLKA) = +ZERZER
      TZPOL  (NPFLKA) = +ZERZER
*  Particle coordinates
      XFLK   (NPFLKA) = RAND() * 0.1 - .05
      YFLK   (NPFLKA) = RAND() * 0.1 - .05
      ZFLK   (NPFLKA) = -0.5
******************************************
*  Changed to 1mm x 1mm section          *
*  Problem with reading FLUKA variables  *
*  from input file                       *
******************************************
*      XFLK   (NPFLKA) = XBEAM
*      YFLK   (NPFLKA) = YBEAM
*      ZFLK   (NPFLKA) = ZBEAM
*  Calculate the total kinetic energy of the primaries: don't change
      IF ( ILOFLK (NPFLKA) .EQ. -2 .OR. ILOFLK (NPFLKA) .GT. 100000 )
     &   THEN
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      ELSE IF ( ILOFLK (NPFLKA) .NE. 0 ) THEN
         TKESUM = TKESUM + ( TKEFLK (NPFLKA) + AMDISC (ILOFLK(NPFLKA)) )
     &          * WTFLK (NPFLKA)
      ELSE
         TKESUM = TKESUM + TKEFLK (NPFLKA) * WTFLK (NPFLKA)
      END IF
      RADDLY (NPFLKA) = ZERZER
*  Here we ask for the region number of the hitting point.
*     NREG (NPFLKA) = ...
*  The following line makes the starting region search much more
*  robust if particles are starting very close to a boundary:
      CALL GEOCRS ( TXFLK (NPFLKA), TYFLK (NPFLKA), TZFLK (NPFLKA) )
      CALL GEOREG ( XFLK  (NPFLKA), YFLK  (NPFLKA), ZFLK  (NPFLKA),
     &              NRGFLK(NPFLKA), IDISC )
*  Do not change these cards:
      CALL GEOHSM ( NHSPNT (NPFLKA), 1, -11, MLATTC )
      NLATTC (NPFLKA) = MLATTC
      CMPATH (NPFLKA) = ZERZER
      CALL SOEVSV
      RETURN
*=== End of subroutine Source =========================================*
      END
